package de.kif.frontend.views.calendar

import de.kif.frontend.launch
import de.kif.frontend.repository.RoomRepository
import de.kif.frontend.repository.ScheduleRepository
import de.westermann.kwebview.View
import de.westermann.kwebview.createHtmlView
import de.westermann.kwebview.iterator
import org.w3c.dom.*
import kotlin.browser.document
import kotlin.browser.window


class Calendar(calendar: HTMLElement) : View(calendar) {

    val day: Int = calendar.dataset["day"]?.toIntOrNull() ?: -1

    private val htmlTag = document.body as HTMLElement
    val calendarTable = calendar.getElementsByClassName("calendar-table")[0] as HTMLElement
    val calendarTableHeader = calendar.getElementsByClassName("calendar-header")[0] as HTMLElement

    fun scrollVerticalBy(pixel: Double, scrollBehavior: ScrollBehavior = ScrollBehavior.SMOOTH) {
        htmlTag.scrollBy(ScrollToOptions(0.0, pixel, scrollBehavior))
    }

    fun scrollHorizontalBy(pixel: Double, scrollBehavior: ScrollBehavior = ScrollBehavior.SMOOTH) {
        calendarTable.scrollBy(ScrollToOptions(pixel, 0.0, scrollBehavior))
    }

    fun scrollVerticalTo(pixel: Double, scrollBehavior: ScrollBehavior = ScrollBehavior.SMOOTH) {
        htmlTag.scrollTo(ScrollToOptions(0.0, pixel, scrollBehavior))
    }

    fun scrollHorizontalTo(pixel: Double, scrollBehavior: ScrollBehavior = ScrollBehavior.SMOOTH) {
        calendarTable.scrollTo(ScrollToOptions(pixel, 0.0, scrollBehavior))
    }

    val editable = calendar.dataset["editable"]?.toBoolean() ?: false

    val body = CalendarBody(this, calendar.getElementsByClassName("calendar-body")[0] as HTMLElement)

    init {
        if (editable) {
            CalendarEdit(this, calendar.querySelector(".calendar-edit") as HTMLElement)
        }

        (document.getElementById("calendar-check-constraints") as? HTMLElement)?.let { wrap(it) }
            ?.onClick?.addListener {
            launch {
                val errors = ScheduleRepository.checkConstraints()

                for ((s, l) in errors.map) {
                    for (entry in body.calendarEntries) {
                        if (entry.scheduleId == s) {
                            entry.error = l.isNotEmpty()
                        }
                    }
                }
            }
        }

        onWheel {
            val multiplier = when (it.deltaMode) {
                1 -> 16.0
                2 -> window.innerHeight.toDouble()
                else -> 1.0
            }

            scrollHorizontalBy(it.deltaX * multiplier, ScrollBehavior.INSTANT)
            scrollVerticalBy(it.deltaY * multiplier, ScrollBehavior.INSTANT)

            it.preventDefault()
        }

        RoomRepository.onCreate {
            val cell = createHtmlView<HTMLElement>()
            cell.dataset["room"] = it.toString()
            cell.classList.add("calendar-cell")
            calendarTableHeader.appendChild(cell)

            launch {
                val room = RoomRepository.get(it) ?: return@launch
                val span = createHtmlView<HTMLSpanElement>()
                span.textContent = room.name
                cell.appendChild(span)
            }
        }

        RoomRepository.onDelete {
            val str = it.toString()
            for (element in calendarTableHeader.children.iterator()) {
                if (element.dataset["room"] == str) {
                    element.remove()
                }
            }
        }
    }
}

fun initCalendar() {
    Calendar(document.getElementsByClassName("calendar")[0] as? HTMLElement ?: return)
}